<?php
//スクリプト作成：https://note.com/tekito_note
//何か不具合があった場合はご連絡ください。
header('Content-Type: application/json');
header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
header('Pragma: no-cache');

require_once 'config.php';

$reactions_file = __DIR__ . '/reactions.csv';
$user_ip = $_SERVER['REMOTE_ADDR'];

if (isIPBlocked($user_ip)) {
    http_response_code(403);
    echo json_encode(['success' => false, 'message' => 'このIPアドレスからのアクセスは制限されています']);
    exit;
}

if (!file_exists($reactions_file)) {
    if (!is_writable(dirname($reactions_file))) {
        error_log('リアクションファイルのディレクトリに書き込み権限がありません');
        echo json_encode(['success' => false, 'message' => 'システムエラーが発生しました']);
        exit;
    }
    // ファイルを作成
    if (!touch($reactions_file)) {
        error_log('リアクションファイルの作成に失敗しました');
        echo json_encode(['success' => false, 'message' => 'システムエラーが発生しました']);
        exit;
    }
    chmod($reactions_file, 0666);
}

if (!is_writable($reactions_file)) {
    error_log('リアクションファイルに書き込み権限がありません');
    echo json_encode(['success' => false, 'message' => 'システムエラーが発生しました']);
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $data = json_decode(file_get_contents('php://input'), true);
    if (!isset(REACTION_CONFIG['reactions'][$data['reaction_type']])) {
        echo json_encode(['success' => false, 'message' => '無効なリアクションタイプです']);
        exit;
    }
    
    $reactions = file_exists($reactions_file) ? array_map('str_getcsv', file($reactions_file)) : [];
    $user_reactions = array_filter($reactions, fn($row) => 
        $row[0] === $user_ip && 
        $row[2] === $data['reaction_id'] && 
        $row[3] === $data['reaction_type']
    );

    $today = date('Y-m-d');
    if (REACTION_CONFIG['limits']['daily']) {
        $today_reactions = array_filter($user_reactions, fn($row) => $row[1] === $today);
        if (count($today_reactions) >= REACTION_CONFIG['limits']['dailyLimit']) {
            echo json_encode(['success' => false]);
            exit;
        }
    } elseif (count($user_reactions) >= REACTION_CONFIG['limits']['perIP']) {
        echo json_encode(['success' => false]);
        exit;
    }
    
    $reactions[] = [$user_ip, $today, $data['reaction_id'], $data['reaction_type']];
    file_put_contents($reactions_file, implode("\n", array_map(fn($row) => implode(',', $row), $reactions)) . "\n");
    
    echo json_encode(['success' => true]);
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'GET') {
    $counts = array_fill_keys(array_keys(REACTION_CONFIG['reactions']), 0);
    
    if (file_exists($reactions_file)) {
        $reactions = array_map('str_getcsv', file($reactions_file));
        foreach ($reactions as $row) {
            if ($row[2] == $_GET['reaction_id'] && isset($counts[$row[3]])) {
                $counts[$row[3]]++;
            }
        }
    }
    
    echo json_encode($counts);
}

